"""
krb5 configuration
==================
The krb5 files are normally available to rules as a list of
Krb5Configuration objects.
"""
from copy import deepcopy
from insights.core import LegacyItemAccess
from insights.core.plugins import combiner
from insights.parsers.krb5 import Krb5Configuration


@combiner(Krb5Configuration)
class AllKrb5Conf(LegacyItemAccess):
    """
    Combiner for accessing all the krb5 configuration files, the format is dict.
    There may be multi files for krb5 configuration, and the main config file is
    krb5.conf. In the situation that same section is both in krb5.conf and other
    configuration files, section in krb5.conf is the available setting. Data from
    parser krb5 is list of dict(s), this combiner will parse this list and return
    a dict which containing all valid data.

    Sample files::

        /etc/krb5.conf:

            includedir /etc/krb5.conf.d/
            include /etc/krb5test.conf
            module /etc/krb5test.conf:residual

            [logging]
                default = FILE:/var/log/krb5libs.log
                kdc = FILE:/var/log/krb5kdc.log

        /etc/krb5.d/krb5_more.conf:

            [logging]
                default = FILE:/var/log/krb5.log
                kdc = FILE:/var/log/krb5.log
                admin_server = FILE:/var/log/kadmind.log

            [realms]
                dns_lookup_realm = false
                default_ccache_name = KEYRING:persistent:%{uid}

    Examples:
        >>> all_krb5 = shared[AllKrb5Conf]
        >>> all_krb5.include
        ['/etc/krb5test.conf']
        >>> all_krb5.sections()
        ['logging', 'realms']
        >>> all_krb5.options('logging')
        ['default', 'kdc', 'admin_server']
        >>> all_krb5['logging']['kdc']
        'FILE:/var/log/krb5kdc.log'
        >>> all_krb5.has_option('logging', 'admin_server')
        True
        >>> all_krb5['realms']['dns_lookup_realm']
        'false'
        >>> all_krb5.files
        ['krb5.conf', 'test.conf', 'test2.conf']

    Attributes:
        includedir (list): The directory list that `krb5.conf` includes via
            `includedir` directive
        include (list): The configuration file list that `krb5.conf` includes
            via `include` directive
        module (list): The module list that `krb5.conf` specifed via 'module'
            directive
        files (list): The list of configuration file names.

    """
    def __init__(self, krb5configs):
        self.data = {}
        main_data = {}
        self.includedir = []
        self.include = []
        self.module = []
        self.files = []

        for krb5_parser in krb5configs:
            self.files.append(krb5_parser.file_name)
            if krb5_parser.file_path == "/etc/krb5.conf":
                main_data = krb5_parser.data
                self.includedir = krb5_parser.includedir
                self.include = krb5_parser.include
                self.module = krb5_parser.module
            else:
                dict_deep_merge(self.data, krb5_parser.data)
        # Same options in same section from other configuration files will be covered by the option
        # from main configuration, but different options in same section will be kept.
        for key, value in main_data.items():
            if key in self.data.keys():
                self.data[key].update(value)
            else:
                self.data[key] = value

        super(AllKrb5Conf, self).__init__()

    def sections(self):
        """
        Return a list of section names.
        """
        return self.data.keys()

    def has_section(self, section):
        """
        Indicate whether the named section is present in the configuration.
        Return True if the given section is present, and False if not present.
        """
        return section in self.data

    def options(self, section):
        """
        Return a list of option names for the given section name.
        """
        return self.data[section].keys() if self.has_section(section) else []

    def has_option(self, section, option):
        """
        Check for the existence of a given option in a given section.
        Return True if the given option is present, and False if not present.
        """
        if section not in self.data:
            return False
        return option in self.data[section]


def dict_deep_merge(tgt, src):
    """
    Utility function to merge the source dictionary `src` to the target
    dictionary recursively

    Note:
        The type of the values in the dictionary can only be `dict` or `list`

    Parameters:
        tgt (dict): The target dictionary
        src (dict): The source dictionary
    """
    for k, v in src.items():
        if k in tgt:
            if isinstance(tgt[k], dict) and isinstance(v, dict):
                dict_deep_merge(tgt[k], v)
            else:
                tgt[k].extend(deepcopy(v))
        else:
            tgt[k] = deepcopy(v)



"""
OpenShiftHosts - file ``/root/.config/openshift/hosts``
========================================================

OpenShiftHosts file is /root/.config/openshift/hosts which
records nodes information. While installing openshift cluster
, this installation process would read this file, and install
relative rpms on every node according to the configuration.
"""


from .. import parser, Parser, get_active_lines, LegacyItemAccess
from insights.parsers import ParseException
from insights.specs import Specs
from insights.core.filters import add_filter

add_filter(Specs.openshift_hosts, "[")


@parser(Specs.openshift_hosts)
class OpenShiftHosts(Parser, LegacyItemAccess):
    """
    Class ``OpenShiftHosts`` parses the content of the ``/root/.config/openshift/hosts``` file.
    A small sample of the content of this file looks like::

        [OSEv3:children]
        nodes
        nfs
        masters
        etcd
        [OSEv3:vars]
        openshift_master_cluster_public_hostname=None
        ansible_ssh_user=root
        openshift_master_cluster_hostname=None
        openshift_hostname_check=false
        deployment_type=openshift-enterprise
        [nodes]
        master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com openshift_schedulable=False ansible_connection=local
        node1.ose35.com  openshift_public_ip=192.66.208.169 openshift_ip=192.66.208.169 openshift_public_hostname=node1.ose35.com openshift_hostname=node1.ose35.com connect_to=node1.ose35.com openshift_node_labels="{'region': 'infra'}" openshift_schedulable=True
        node2.ose35.com  openshift_public_ip=192.66.208.170 openshift_ip=192.66.208.170 openshift_public_hostname=node2.ose35.com openshift_hostname=node2.ose35.com connect_to=node2.ose35.com openshift_node_labels="{'region': 'infra'}" openshift_schedulable=True
        [nfs]
        master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com ansible_connection=local
        [masters]
        master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com ansible_connection=local
        [etcd]
        master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com ansible_connection=local

    Examples:
        >>> type(host_info)
        <class 'insights.parsers.openshift_hosts.OpenShiftHosts'>
        >>> host_info["OSEv3:children"]
        ["nodes","nfs","masters","etcd"]
        >>> host_info["nodes"]["master.ose35.com"]["openshift_public_ip"]
        "192.66.208.202"
        >>> host_info["nodes"]["master.ose35.com"]["openshift_node_labels"]
        "{'region': 'infra'}"
        >>> host_info.has_node("node1.ose35.com")
        True
        >>> host_info.has_var("openshift_use_crio")
        False

    """

    def parse_content(self, content):
        dict_all = {}
        sub_section = ""
        for line in get_active_lines(content):
            if line.startswith("[") and line.endswith("]"):
                sub_section = line[1:-1].strip()
                if "OSEv3:children" in sub_section:
                    dict_all[sub_section] = []
                else:
                    dict_all[sub_section] = {}
                continue
            else:
                # Parser section [OSEv3:children]
                if "OSEv3:children" in sub_section:
                    dict_all[sub_section].append(line)
                # Parser section [OSEv3:vars]
                elif "OSEv3:vars" in sub_section:
                    key, value = line.split("=")
                    dict_all[sub_section][key.strip()] = value.strip()
                else:
                    nodename, nodeinfo = line.split(" ", 1)
                    dict_all[sub_section][nodename] = {}
                    splitwithsqual = nodeinfo.split("=")
                    keys = [splitwithsqual[0].strip()]
                    values = []
                    for item in splitwithsqual[1:-1]:
                        value, key = item.rsplit(" ", 1)
                        keys.append(key.strip())
                        # There is key-value like "openshift_node_labels="{'region': 'infra','zone': 'default'}"",
                        # convert the value as dict.
                        if "{" and "}" in value:
                            subkeyvalue = value[2:-2].split(",")
                            subdict = {}
                            for subitem in subkeyvalue:
                                subkey, subvalue = subitem.split(":")
                                subdict[subkey.strip()[1:-1]] = subvalue.strip()[1:-1]
                            values.append(subdict)
                        else:
                            values.append(value.strip())
                    values.append(splitwithsqual[-1].strip())
                    if len(keys) != len(values):
                        raise ParseException("Wrong Configuration")
                    for i in range(len(keys)):
                        dict_all[sub_section][nodename][keys[i]] = values[i]
        self.data = dict_all

    def has_var(self, var):
        """
        Indicate whether the named var is present in the configuration.
        Return True if the given var is present, and False if not present.
        """
        return "OSEv3:vars" in self.data and var in self.data["OSEv3:vars"]

    def has_node(self, node):
        """
        Indicate whether the named node is present in the configuration.
        Return True if the given node name is present, and False if not present.
        """
        return "nodes" in self.data and node in self.data["nodes"]

    def has_node_type(self, node_type):
        """
        Indicate whether the named node type is present in the configuration.
        Return True if the given node type is present, and False if not present.
        """
        return "OSEv3:children" in self.data and node_type in self.data["OSEv3:children"]



from insights.parsers.softnet_stat import SoftNetStats
from insights.tests import context_wrap

SOFTNET_STAT = """
00008e78 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
000040ee 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
0001608c 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
0000372f 00000001 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
"""

SOFTNET_STAT_2 = """
00008e78 00000000 000000011 00000000 00000000 00000000 00000000 00000000 00000020 00000010
000040ee 00000000 00000000 00000000 00000000 00000000 00000000 00000000 000000010 00000001
"""
SOFTNET_STAT_NO = """
"""


def test_softnet_stat():
    stats = SoftNetStats(context_wrap(SOFTNET_STAT))
    assert stats.cpu_instances == 4
    assert stats.is_packet_drops
    assert stats is not None

    assert stats.per_cpu_nstat('packet_drops') == [0, 0, 0, 1]
    assert stats.per_cpu_nstat('time_squeeze') == [0, 0, 0, 0]
    assert stats.per_cpu_nstat('cpu_collision') == [0, 0, 0, 0]
    assert stats.per_cpu_nstat('received_rps') == [0, 0, 0, 0]
    assert stats.per_cpu_nstat('flow_limit_count') == [0, 0, 0, 0]

    stats = SoftNetStats(context_wrap(SOFTNET_STAT_NO))
    assert stats is not None
    assert stats.cpu_instances == 0
    assert not stats.is_packet_drops
    assert stats.per_cpu_nstat('packet_drops') == []
    assert stats.per_cpu_nstat('time_squeeze') == []
    assert stats.per_cpu_nstat('cpu_collision') == []
    assert stats.per_cpu_nstat('received_rps') == []
    assert stats.per_cpu_nstat('flow_limit_count') == []

    stats = SoftNetStats(context_wrap(SOFTNET_STAT_2))
    assert stats is not None
    assert stats.cpu_instances == 2
    assert stats.per_cpu_nstat('packet_drops') == [0, 0]
    assert stats.per_cpu_nstat('time_squeeze') == [17, 0]
    assert stats.per_cpu_nstat('cpu_collision') == [0, 0]
    assert stats.per_cpu_nstat('received_rps') == [32, 16]
    assert stats.per_cpu_nstat('flow_limit_count') == [16, 1]
    assert stats.per_cpu_nstat('packet_dr') == []
    assert not stats.is_packet_drops



from insights.parsers import catalina_log
from insights.parsers.catalina_log import CatalinaServerLog, CatalinaOut
from insights.tests import context_wrap
from datetime import datetime
import doctest

SERVER_LOG = """
INFO: Command line argument: -Djava.io.tmpdir=/var/cache/tomcat/temp
Nov 28, 2017 2:11:20 PM org.apache.catalina.startup.VersionLoggerListener log
INFO: Command line argument: -Djava.util.logging.config.file=/usr/share/tomcat/conf/logging.properties
Nov 28, 2017 2:11:20 PM org.apache.catalina.startup.VersionLoggerListener log
INFO: Command line argument: -Djava.util.logging.manager=org.apache.juli.ClassLoaderLogManager
Nov 28, 2017 2:11:20 PM org.apache.catalina.core.AprLifecycleListener lifecycleEvent
INFO: The APR based Apache Tomcat Native library which allows optimal performance in production environments was not found on the java.library.path: /usr/java/packages/lib/amd64:/usr/lib64:/lib64:/lib:/usr/lib
Nov 28, 2017 2:11:22 PM org.apache.coyote.AbstractProtocol init
INFO: Initializing ProtocolHandler ["http-bio-18080"]
Nov 28, 2017 2:11:23 PM org.apache.coyote.AbstractProtocol init
SEVERE: Failed to initialize end point associated with ProtocolHandler ["http-bio-18080"]
""".strip()


def test_catalina_server_log():
    ser_log = CatalinaServerLog(context_wrap(SERVER_LOG))
    test_1 = ser_log.get('-Djava')
    assert 3 == len(test_1)
    test_2 = ser_log.get('Failed to initialize')
    assert 1 == len(test_2)
    assert test_2[0]['raw_message'] == 'SEVERE: Failed to initialize end point associated with ProtocolHandler ["http-bio-18080"]'
    assert len(list(ser_log.get_after(datetime(2017, 11, 28, 14, 11, 21)))) == 4
    assert "/var/cache/tomcat/temp" in ser_log


CATALINA_OUT1 = """
Nov 10, 2015 8:52:38 AM org.apache.jk.common.MsgAjp processHeader
SEVERE: BAD packet signature 18245
Nov 10, 2015 8:52:38 AM org.apache.jk.common.ChannelSocket processConnection
SEVERE: Error, processing connection
SEVERE: BAD packet signature 18245
Nov 10, 2015 8:52:38 AM org.apache.jk.common.ChannelSocket processConnection
SEVERE: Error, processing connection
INFO: Pausing Coyote HTTP/1.1 on http-8080
Nov 10, 2015 4:55:48 PM org.apache.coyote.http11.Http11Protocol pause
INFO: Pausing Coyote HTTP/1.1 on http-8443
Nov 10, 2015 4:55:49 PM org.apache.catalina.core.StandardService stop
INFO: Stopping service Catalina
INFO: Initializing Coyote HTTP/1.1 on http-8080
Nov 10, 2015 6:09:39 PM org.apache.coyote.http11.Http11Protocol init
INFO: Initializing Coyote HTTP/1.1 on http-8443
Nov 10, 2015 6:09:39 PM org.apache.catalina.startup.Catalina load
INFO: Initialization processed in 514 ms
Nov 10, 2015 6:09:39 PM org.apache.catalina.core.StandardService start
INFO: Starting service Catalina
Nov 10, 2015 6:09:39 PM org.apache.catalina.core.StandardEngine start
INFO: Starting Servlet Engine: Apache Tomcat/6.0.24
Nov 10, 2015 6:09:39 PM org.apache.catalina.startup.HostConfig deployDescriptor
INFO: Deploying configuration descriptor candlepin.xml
Nov 10, 2015 6:09:39 PM org.apache.catalina.loader.WebappClassLoader validateJarFile
INFO: validateJarFile(/usr/share/tomcat6/webapps/candlepin/WEB-INF/lib/servlet.jar) - jar not loaded. See Servlet Spec 2.3, section 9.7.2. Offending class: javax/servlet/Servlet.class
Nov 10, 2015 6:09:40 PM com.google.inject.internal.ProxyFactory <init>
WARNING: Method [protected org.candlepin.model.AbstractHibernateObject org.candlepin.resource.ConsumerContentOverrideResource.findParentById(java.lang.String)] is synthetic and is being intercepted by [org.candlepin.guice.CandlepinResourceTxnInterceptor@3b967594]. This could indicate a bug.  The method may be intercepted twice, or may not be intercepted at all.
Nov 10, 2015 6:09:40 PM com.google.inject.internal.ProxyFactory <init>
WARNING: Method [protected org.candlepin.model.AbstractHibernateObject org.candlepin.resource.ActivationKeyContentOverrideResource.findParentById(java.lang.String)] is synthetic and is being intercepted by [org.candlepin.guice.CandlepinResourceTxnInterceptor@3b967594]. This could indicate a bug.  The method may be intercepted twice, or may not be intercepted at all.
Nov 10, 2015 6:09:42 PM com.mchange.v2.log.MLog <clinit>
INFO: MLog clients using java 1.4+ standard logging.
Nov 10, 2015 6:09:42 PM com.mchange.v2.c3p0.C3P0Registry banner
INFO: Initializing c3p0-0.9.1.2 [built 31-March-2011 15:45:28; debug? false; trace: 5]
Nov 10, 2015 6:09:42 PM com.mchange.v2.c3p0.impl.AbstractPoolBackedDataSource getPoolManager
INFO: Initializing c3p0 pool... com.mchange.v2.c3p0.PoolBackedDataSource@64caac34 [ connectionPoolDataSource -> com.mchange.v2.c3p0.WrapperConnectionPoolDataSource@423e8640 [ acquireIncrement -> 3, acquireRetryAttempts -> 30, acquireRetryDelay -> 1000, autoCommitOnClose -> false, automaticTestTable -> null, breakAfterAcquireFailure -> false, checkoutTimeout -> 0, connectionCustomizerClassName -> null, connectionTesterClassName -> com.mchange.v2.c3p0.impl.DefaultConnectionTester, debugUnreturnedConnectionStackTraces -> false, factoryClassLocation -> null, forceIgnoreUnresolvedTransactions -> false, identityToken -> 2sonpk9c1vlvek918mzet6|3f7eb8ff, idleConnectionTestPeriod -> 300, initialPoolSize -> 5, maxAdministrativeTaskTime -> 0, maxConnectionAge -> 0, maxIdleTime -> 300, maxIdleTimeExcessConnections -> 0, maxPoolSize -> 20, maxStatements -> 0, maxStatementsPerConnection -> 0, minPoolSize -> 5, nestedDataSource -> com.mchange.v2.c3p0.DriverManagerDataSource@b4aff57d [ description -> null, driverClass -> null, factoryClassLocation -> null, identityToken -> 2sonpk9c1vlvek918mzet6|7a35d387, jdbcUrl -> jdbc:postgresql://localhost:5432/candlepin, properties -> {user=******, password=******, autocommit=true, release_mode=auto} ], preferredTestQuery -> null, propertyCycle -> 0, testConnectionOnCheckin -> false, testConnectionOnCheckout -> false, unreturnedConnectionTimeout -> 0, usesTraditionalReflectiveProxies -> false; userOverrides: {} ], dataSourceName -> null, factoryClassLocation -> null, identityToken -> 2sonpk9c1vlvek918mzet6|2764e21d, numHelperThreads -> 3 ]
WARN [UUIDHexGenerator] - HHH000409: Using org.hibernate.id.UUIDHexGenerator which does not generate IETF RFC 4122 compliant UUID values; consider using org.hibernate.id.UUIDGenerator instead
Nov 10, 2015 6:09:45 PM org.apache.catalina.startup.HostConfig deployDescriptor
INFO: Deploying configuration descriptor gutterball.xml
Nov 10, 2015 6:09:45 PM org.apache.catalina.loader.WebappClassLoader validateJarFile
INFO: validateJarFile(/usr/share/tomcat6/webapps/gutterball/WEB-INF/lib/servlet.jar) - jar not loaded. See Servlet Spec 2.3, section 9.7.2. Offending class: javax/servlet/Servlet.class
WARN [ConnectionProviderInitiator] - HHH000208: org.hibernate.connection.C3P0ConnectionProvider has been deprecated in favor of org.hibernate.service.jdbc.connections.internal.C3P0ConnectionProvider; that provider will be used instead.
Nov 10, 2015 6:09:46 PM com.mchange.v2.log.MLog <clinit>
INFO: MLog clients using java 1.4+ standard logging.
Nov 10, 2015 6:09:46 PM com.mchange.v2.c3p0.C3P0Registry banner
INFO: Initializing c3p0-0.9.1.2 [built 31-March-2011 15:45:28; debug? false; trace: 5]
Nov 10, 2015 6:09:46 PM com.mchange.v2.c3p0.management.ActiveManagementCoordinator attemptManageC3P0Registry
WARNING: A C3P0Registry mbean is already registered. This probably means that an application using c3p0 was undeployed, but not all PooledDataSources were closed prior to undeployment. This may lead to resource leaks over time. Please take care to close all PooledDataSources.
Nov 10, 2015 6:09:46 PM com.mchange.v2.c3p0.impl.AbstractPoolBackedDataSource getPoolManager
INFO: Initializing c3p0 pool... com.mchange.v2.c3p0.PoolBackedDataSource@daaa54fa [ connectionPoolDataSource -> com.mchange.v2.c3p0.WrapperConnectionPoolDataSource@813e3887 [ acquireIncrement -> 3, acquireRetryAttempts -> 30, acquireRetryDelay -> 1000, autoCommitOnClose -> false, automaticTestTable -> null, breakAfterAcquireFailure -> false, checkoutTimeout -> 0, connectionCustomizerClassName -> null, connectionTesterClassName -> com.mchange.v2.c3p0.impl.DefaultConnectionTester, debugUnreturnedConnectionStackTraces -> false, factoryClassLocation -> null, forceIgnoreUnresolvedTransactions -> false, identityToken -> 2sonpk9c1vlvhv6yu2uxw|5adff1b7, idleConnectionTestPeriod -> 300, initialPoolSize -> 5, maxAdministrativeTaskTime -> 0, maxConnectionAge -> 0, maxIdleTime -> 300, maxIdleTimeExcessConnections -> 0, maxPoolSize -> 20, maxStatements -> 0, maxStatementsPerConnection -> 0, minPoolSize -> 5, nestedDataSource -> com.mchange.v2.c3p0.DriverManagerDataSource@a5e5a991 [ description -> null, driverClass -> null, factoryClassLocation -> null, identityToken -> 2sonpk9c1vlvhv6yu2uxw|4d06026b, jdbcUrl -> jdbc:postgresql:gutterball, properties -> {user=******, password=******, autocommit=true, release_mode=auto} ], preferredTestQuery -> null, propertyCycle -> 0, testConnectionOnCheckin -> false, testConnectionOnCheckout -> false, unreturnedConnectionTimeout -> 0, usesTraditionalReflectiveProxies -> false; userOverrides: {} ], dataSourceName -> null, factoryClassLocation -> null, identityToken -> 2sonpk9c1vlvhv6yu2uxw|7efd82a6, numHelperThreads -> 3 ]
WARN [UUIDHexGenerator] - HHH000409: Using org.hibernate.id.UUIDHexGenerator which does not generate IETF RFC 4122 compliant UUID values; consider using org.hibernate.id.UUIDGenerator instead
Nov 10, 2015 6:09:47 PM org.apache.coyote.http11.Http11Protocol start
INFO: Starting Coyote HTTP/1.1 on http-8080
Nov 10, 2015 6:09:47 PM org.apache.coyote.http11.Http11Protocol start
INFO: Starting Coyote HTTP/1.1 on http-8443
Nov 10, 2015 6:09:47 PM org.apache.jk.common.ChannelSocket init
INFO: JK: ajp13 listening on /0.0.0.0:8009
Nov 10, 2015 6:09:47 PM org.apache.jk.server.JkMain start
INFO: Jk running ID=0 time=0/15  config=null
Nov 10, 2015 6:09:47 PM org.apache.catalina.startup.Catalina start
INFO: Server startup in 8434 ms
Nov 10, 2015 6:36:08 PM org.apache.coyote.http11.Http11Protocol pause
INFO: Pausing Coyote HTTP/1.1 on http-8080
Nov 10, 2015 6:36:08 PM org.apache.coyote.http11.Http11Protocol pause
INFO: Pausing Coyote HTTP/1.1 on http-8443
Nov 10, 2015 6:36:09 PM org.apache.catalina.core.StandardService stop
INFO: Stopping service Catalina
WARN [client] - HQ212037: Connection failure has been detected: HQ119015: The connection was disconnected because of server shutdown [code=DISCONNECTED]
WARN [client] - HQ212037: Connection failure has been detected: HQ119015: The connection was disconnected because of server shutdown [code=DISCONNECTED]
WARN [client] - HQ212037: Connection failure has been detected: HQ119015: The connection was disconnected because of server shutdown [code=DISCONNECTED]
WARN [server] - HQ222113: On ManagementService stop, there are 1 unexpected registered MBeans: [core.acceptor.2175e4c3-8800-11e5-9ba9-3f3f219d3ca7]
Nov 10, 2015 6:37:58 PM org.apache.catalina.core.AprLifecycleListener init
INFO: The APR based Apache Tomcat Native library which allows optimal performance in production environments was not found on the java.library.path: /usr/java/packages/lib/amd64:/usr/lib64:/lib64:/lib:/usr/lib
Nov 10, 2015 6:37:59 PM org.apache.coyote.http11.Http11Protocol init
INFO: Initializing Coyote HTTP/1.1 on http-8080
Nov 10, 2015 6:38:00 PM org.apache.coyote.http11.Http11Protocol init
INFO: Initializing Coyote HTTP/1.1 on http-8443
Nov 10, 2015 6:38:00 PM org.apache.catalina.startup.Catalina load
INFO: Initialization processed in 2278 ms
Nov 10, 2015 6:38:00 PM org.apache.catalina.core.StandardService start
INFO: Starting service Catalina
Nov 10, 2015 6:38:00 PM org.apache.catalina.core.StandardEngine start
INFO: Starting Servlet Engine: Apache Tomcat/6.0.24
Nov 10, 2015 6:38:00 PM org.apache.catalina.startup.HostConfig deployDescriptor
INFO: Deploying configuration descriptor candlepin.xml
Nov 10, 2015 6:38:01 PM org.apache.catalina.loader.WebappClassLoader validateJarFile
INFO: validateJarFile(/usr/share/tomcat6/webapps/candlepin/WEB-INF/lib/servlet.jar) - jar not loaded. See Servlet Spec 2.3, section 9.7.2. Offending class: javax/servlet/Servlet.class
Nov 10, 2015 6:38:26 PM com.mchange.v2.c3p0.management.ActiveManagementCoordinator attemptManageC3P0Registry
WARNING: A C3P0Registry mbean is already registered. This probably means that an application using c3p0 was undeployed, but not all PooledDataSources were closed prior to undeployment. This may lead to resource leaks over time. Please take care to close all PooledDataSources.
Nov 10, 2015 6:38:26 PM com.mchange.v2.c3p0.impl.AbstractPoolBackedDataSource getPoolManager
INFO: Initializing c3p0 pool... com.mchange.v2.c3p0.PoolBackedDataSource@86244320 [ connectionPoolDataSource -> com.mchange.v2.c3p0.WrapperConnectionPoolDataSource@d2a45c5e [ acquireIncrement -> 3, acquireRetryAttempts -> 30, acquireRetryDelay -> 1000, autoCommitOnClose -> false, automaticTestTable -> null, breakAfterAcquireFailure -> false, checkoutTimeout -> 0, connectionCustomizerClassName -> null, connectionTesterClassName -> com.mchange.v2.c3p0.impl.DefaultConnectionTester, debugUnreturnedConnectionStackTraces -> false, factoryClassLocation -> null, forceIgnoreUnresolvedTransactions -> false, identityToken -> 2sonpk9c1vmwckfzb7843|3ee3c109, idleConnectionTestPeriod -> 300, initialPoolSize -> 5, maxAdministrativeTaskTime -> 0, maxConnectionAge -> 0, maxIdleTime -> 300, maxIdleTimeExcessConnections -> 0, maxPoolSize -> 20, maxStatements -> 0, maxStatementsPerConnection -> 0, minPoolSize -> 5, nestedDataSource -> com.mchange.v2.c3p0.DriverManagerDataSource@4d38de29 [ description -> null, driverClass -> null, factoryClassLocation -> null, identityToken -> 2sonpk9c1vmwckfzb7843|34039dfc, jdbcUrl -> jdbc:postgresql:gutterball, properties -> {user=******, password=******, autocommit=true, release_mode=auto} ], preferredTestQuery -> null, propertyCycle -> 0, testConnectionOnCheckin -> false, testConnectionOnCheckout -> false, unreturnedConnectionTimeout -> 0, usesTraditionalReflectiveProxies -> false; userOverrides: {} ], dataSourceName -> null, factoryClassLocation -> null, identityToken -> 2sonpk9c1vmwckfzb7843|73f90cb5, numHelperThreads -> 3 ]
WARN [UUIDHexGenerator] - HHH000409: Using org.hibernate.id.UUIDHexGenerator which does not generate IETF RFC 4122 compliant UUID values; consider using org.hibernate.id.UUIDGenerator instead
Nov 10, 2015 6:38:26 PM org.apache.coyote.http11.Http11Protocol start
INFO: Starting Coyote HTTP/1.1 on http-8080
Nov 10, 2015 6:38:26 PM org.apache.coyote.http11.Http11Protocol start
INFO: Starting Coyote HTTP/1.1 on http-8443
Nov 10, 2015 6:38:27 PM org.apache.jk.common.ChannelSocket init
INFO: JK: ajp13 listening on /0.0.0.0:8009
Nov 10, 2015 6:38:27 PM org.apache.jk.server.JkMain start
INFO: Jk running ID=0 time=0/107  config=null
Nov 10, 2015 6:38:27 PM org.apache.catalina.startup.Catalina start
INFO: Server startup in 27028 ms
""".strip()


def test_catalina_out():
    out_log = CatalinaOut(context_wrap(CATALINA_OUT1))
    assert "IETF RFC 4122 compliant UUID" in out_log
    assert len(out_log.get("SEVERE")) == 4
    assert len(list(out_log.get_after(datetime(2015, 11, 10, 18, 38, 10)))) == 15


CATALINA_OUT2 = """
Nov 10, 2015 8:52:38 AM org.apache.jk.common.MsgAjp processHeader
SEVERE: BAD packet signature 18245
Nov 10, 2015 8:52:38 AM org.apache.jk.common.ChannelSocket processConnection
SEVERE: Error, processing connection
SEVERE: BAD packet signature 18245
Nov 10, 2015 8:52:38 AM org.apache.jk.common.ChannelSocket processConnection
SEVERE: Error, processing connection
Nov 10, 2015 4:55:48 PM org.apache.coyote.http11.Http11Protocol pause
INFO: Pausing Coyote HTTP/1.1 on http-8080
""".strip()


def test_catalina_log_doc_examples():
    env = {
            'CatalinaOut': CatalinaOut,
            'out': CatalinaOut(context_wrap(CATALINA_OUT2, path='/var/log/tomcat/catalina.out')),
            'log': CatalinaServerLog(context_wrap(SERVER_LOG, path='/var/log/tomcat/catalina.2017-11-28.log'))
          }
    failed, total = doctest.testmod(catalina_log, globs=env)
    assert failed == 0



import doctest

from insights.combiners import lspci
from insights.combiners.lspci import LsPci
from insights.parsers.lspci import LsPci as LsPciParser, LsPciVmmkn
from insights.tests import context_wrap


LSPCI_K = """
00:00.0 Host bridge: Intel Corporation Haswell-ULT DRAM Controller (rev 09)
        Subsystem: Lenovo ThinkPad X240
        Kernel driver in use: hsw_uncore
00:02.0 VGA compatible controller: Intel Corporation Haswell-ULT Integrated Graphics Controller (rev 09)
        Subsystem: Lenovo ThinkPad X240
        Kernel driver in use: i915
        Kernel modules: i915
00:03.0 Audio device: Intel Corporation Haswell-ULT HD Audio Controller (rev 09)
        Subsystem: Lenovo ThinkPad X240
        Kernel driver in use: snd_hda_intel
        Kernel modules: snd_hda_intel
00:16.0 Communication controller: Intel Corporation 8 Series HECI #0 (rev 04)
        Subsystem: Lenovo ThinkPad X240
        Kernel driver in use: mei_me
        Kernel modules: mei_me
00:19.0 Ethernet controller: Intel Corporation Ethernet Connection I218-LM (rev 04)
        Subsystem: Lenovo ThinkPad X240
        Kernel driver in use: e1000e
        Kernel modules: e1000e
00:1b.0 Audio device: Intel Corporation 8 Series HD Audio Controller (rev 04)
        Subsystem: Lenovo ThinkPad X240
        Kernel driver in use: snd_hda_intel
        Kernel modules: snd_hda_intel
""".strip()

LSPCI_VMMKN = """

Slot:   00:00.0
Class:  0600
Vendor: 8086
Device: 0a04
SVendor:        17aa
SDevice:        2214
Rev:    09
Driver: hsw_uncore

Slot:   00:02.0
Class:  0300
Vendor: 8086
Device: 0a16
SVendor:        17aa
SDevice:        2214
Rev:    09
Driver: i915
Module: i915

Slot:   00:03.0
Class:  0403
Vendor: 8086
Device: 0a0c
SVendor:        17aa
SDevice:        2214
Rev:    09
Driver: snd_hda_intel
Module: snd_hda_intel

Slot:   00:16.0
Class:  0780
Vendor: 8086
Device: 9c3a
SVendor:        17aa
SDevice:        2214
Rev:    04
Driver: mei_me
Module: mei_me


Slot:   00:19.0
Class:  0200
Vendor: 8086
Device: 155a
SVendor:        17aa
SDevice:        2214
Rev:    04
Driver: e1000e
Module: e1000e

Slot:   00:1b.0
Class:  0403
Vendor: 8086
Device: 9c20
SVendor:        17aa
SDevice:        2214
Rev:    04
Driver: snd_hda_intel
Module: snd_hda_intel
"""

LSPCI_K_LONG_SLOT = """
0000:00:00.0 Host bridge: Intel Corporation 440BX/ZX/DX - 82443BX/ZX/DX Host bridge (AGP disabled) (rev 03)
0000:00:07.0 ISA bridge: Intel Corporation 82371AB/EB/MB PIIX4 ISA (rev 01)
        Subsystem: Microsoft Corporation Device 0000
0000:00:07.1 IDE interface: Intel Corporation 82371AB/EB/MB PIIX4 IDE (rev 01)
        Kernel driver in use: ata_piix
        Kernel modules: ata_piix, pata_acpi, ata_generic
0000:00:07.3 Bridge: Intel Corporation 82371AB/EB/MB PIIX4 ACPI (rev 02)
        Kernel modules: i2c_piix4
0000:00:08.0 VGA compatible controller: Microsoft Corporation Hyper-V virtual VGA
        Kernel driver in use: hyperv_fb
        Kernel modules: hyperv_fb
1a06:00:02.0 Ethernet controller: Mellanox Technologies MT27710 Family [ConnectX-4 Lx Virtual Function] (rev 80)
        Subsystem: Mellanox Technologies Device 0190
        Kernel driver in use: mlx5_core
        Kernel modules: mlx5_core
"""

LSPCI_VMMKN_LONG_SLOT = """
Slot:   00:00.0
Class:  0600
Vendor: 8086
Device: 7192
Rev:    03

Slot:   00:07.0
Class:  0601
Vendor: 8086
Device: 7110
SVendor:        1414
SDevice:        0000
Rev:    01

Slot:   00:07.1
Class:  0101
Vendor: 8086
Device: 7111
Rev:    01
ProgIf: 80
Driver: ata_piix
Module: ata_piix
Module: pata_acpi
Module: ata_generic

Slot:   00:07.3
Class:  0680
Vendor: 8086
Device: 7113
Rev:    02
Module: i2c_piix4

Slot:   00:08.0
Class:  0300
Vendor: 1414
Device: 5353
Driver: hyperv_fb
Module: hyperv_fb

Slot:   1a06:00:02.0
Class:  0200
Vendor: 15b3
Device: 1016
SVendor:        15b3
SDevice:        0190
PhySlot:        1
Rev:    80
Driver: mlx5_core
Module: mlx5_core
NUMANode:       0
"""


def test_lspci_k():
    lspci_k = LsPciParser(context_wrap(LSPCI_K))
    lspci = LsPci(lspci_k, None)
    assert sorted(lspci.pci_dev_list) == ['00:00.0', '00:02.0', '00:03.0', '00:16.0', '00:19.0', '00:1b.0']
    assert lspci.search(Dev_Details__contains='I218') == [
        {
            'Slot': '00:19.0',
            'Driver': 'e1000e', 'Module': ['e1000e'],
            'Subsystem': 'Lenovo ThinkPad X240',
            'Dev_Details': 'Ethernet controller: Intel Corporation Ethernet Connection I218-LM (rev 04)'
        }
    ]
    assert lspci.search(Slot__startwith='00:1b.0') == []
    # Make sure the original parser is untouched
    assert lspci_k.pci_dev_details('00:00.0').get('Kernel driver in use') == 'hsw_uncore'
    assert lspci_k.pci_dev_details('00:1b.0').get('Kernel driver in use') == 'snd_hda_intel'


def test_lspci_vmmkn():
    lspci_vmmkn = LsPciVmmkn(context_wrap(LSPCI_VMMKN))
    lspci = LsPci(None, lspci_vmmkn)
    assert sorted(lspci.pci_dev_list) == ['00:00.0', '00:02.0', '00:03.0', '00:16.0', '00:19.0', '00:1b.0']
    assert lspci.search(Device='155a', Vendor='8086') == [
        {
            'Slot': '00:19.0', 'Class': '0200', 'Vendor': '8086',
            'Device': '155a', 'SVendor': '17aa', 'SDevice': '2214',
            'Rev': '04', 'Driver': 'e1000e', 'Module': ['e1000e'],
        }
    ]
    assert lspci.search(Dev_Details__contains='I218') == []
    # Make sure the original parser is untouched
    assert sorted(lspci_vmmkn[0].keys()) == sorted(['Slot', 'Class', 'Vendor',
                        'Device', 'SVendor', 'SDevice', 'Rev', 'Driver'])


def test_lspci_both():
    lspci_vmmkn = LsPciVmmkn(context_wrap(LSPCI_VMMKN))
    lspci_k = LsPciParser(context_wrap(LSPCI_K))
    lspci = LsPci(lspci_k, lspci_vmmkn)
    assert sorted(lspci.pci_dev_list) == ['00:00.0', '00:02.0', '00:03.0', '00:16.0', '00:19.0', '00:1b.0']
    assert lspci.search(Dev_Details__contains='I218') == [
        {
            'Slot': '00:19.0', 'Class': '0200', 'Vendor': '8086',
            'Device': '155a', 'SVendor': '17aa', 'SDevice': '2214',
            'Rev': '04', 'Driver': 'e1000e', 'Module': ['e1000e'],
            'Subsystem': 'Lenovo ThinkPad X240',
            'Dev_Details': 'Ethernet controller: Intel Corporation Ethernet Connection I218-LM (rev 04)'
        }
    ]
    assert lspci.search(Slot='00:1b.0') == [
        {
            'Slot': '00:1b.0', 'Class': '0403', 'Vendor': '8086',
            'Device': '9c20', 'SVendor': '17aa', 'SDevice': '2214',
            'Rev': '04', 'Driver': 'snd_hda_intel',
            'Module': ['snd_hda_intel'],
            'Subsystem': 'Lenovo ThinkPad X240',
            'Dev_Details': 'Audio device: Intel Corporation 8 Series HD Audio Controller (rev 04)'
        }
    ]
    # Make sure the original parsers are untouched
    assert lspci_k.pci_dev_details('00:00.0').get('Kernel driver in use') == 'hsw_uncore'
    assert lspci_k.pci_dev_details('00:1b.0').get('Kernel driver in use') == 'snd_hda_intel'
    assert sorted(lspci_vmmkn[0].keys()) == sorted(['Slot', 'Class', 'Vendor',
            'Device', 'SVendor', 'SDevice', 'Rev', 'Driver'])
    assert sorted(lspci_vmmkn[-1].keys()) == sorted(['Class', 'Device',
            'Driver', 'Module', 'Rev', 'SDevice', 'SVendor', 'Slot', 'Vendor'])


def test_lspci_both_long_slot():
    lspci_vmmkn = LsPciVmmkn(context_wrap(LSPCI_VMMKN_LONG_SLOT))
    lspci_k = LsPciParser(context_wrap(LSPCI_K_LONG_SLOT))
    lspci = LsPci(lspci_k, lspci_vmmkn)
    assert sorted(lspci.pci_dev_list) == ['0000:00:00.0', '0000:00:07.0', '0000:00:07.1', '0000:00:07.3', '0000:00:08.0', '1a06:00:02.0']
    assert lspci.search(Dev_Details__contains='PIIX4 ISA') == [
        {
            'Slot': '0000:00:07.0', 'Class': '0601', 'Vendor': '8086',
            'Device': '7110', 'SVendor': '1414', 'SDevice': '0000',
            'Rev': '01',
            'Subsystem': 'Microsoft Corporation Device 0000',
            'Dev_Details': 'ISA bridge: Intel Corporation 82371AB/EB/MB PIIX4 ISA (rev 01)'
        }
    ]
    assert lspci.search(Slot='1a06:00:02.0') == [
        {
            'Slot': '1a06:00:02.0', 'Class': '0200', 'Vendor': '15b3',
            'Device': '1016', 'SVendor': '15b3', 'SDevice': '0190',
            'Rev': '80', 'Driver': 'mlx5_core', 'PhySlot': '1',
            'Module': ['mlx5_core'], 'NUMANode': '0',
            'Subsystem': 'Mellanox Technologies Device 0190',
            'Dev_Details': 'Ethernet controller: Mellanox Technologies MT27710 Family [ConnectX-4 Lx Virtual Function] (rev 80)'
        }
    ]
    # Make sure the original parsers are untouched
    assert sorted(lspci_k.pci_dev_list) == ['0000:00:00.0', '0000:00:07.0',
            '0000:00:07.1', '0000:00:07.3', '0000:00:08.0', '1a06:00:02.0']
    assert lspci_k.pci_dev_details('0000:00:00.0').get('Module') is None
    assert lspci_k.pci_dev_details('0000:00:08.0').get('Kernel driver in use') == 'hyperv_fb'
    assert sorted(lspci_vmmkn.pci_dev_list) == ['00:00.0', '00:07.0', '00:07.1', '00:07.3', '00:08.0', '1a06:00:02.0']
    assert sorted(lspci_vmmkn[0].keys()) == sorted(['Class', 'Device', 'Rev', 'Slot', 'Vendor'])
    assert sorted(lspci_vmmkn[-1].keys()) == sorted(['Class', 'Device',
            'Driver', 'Module', 'Rev', 'SDevice', 'SVendor', 'Slot', 'Vendor',
            'PhySlot', 'NUMANode'])


def test_doc_examples():
    lspci_vmmkn = LsPciVmmkn(context_wrap(LSPCI_VMMKN))
    lspci_k = LsPciParser(context_wrap(LSPCI_K))
    env = {'lspci': LsPci(lspci_k, lspci_vmmkn)}
    failed, total = doctest.testmod(lspci, globs=env)
    assert failed == 0
